﻿using System;

using DWORD = System.UInt32;
using LARGE_INTEGER = System.UInt64;

namespace Microsoft.Win32
{
	/// <summary>
	/// Represents the status of a file copy.
	/// </summary>
	public sealed class CopyFileProgress
	{
		/// <summary>
		/// The <see cref="CopyFileProgressCallback"/> method the user wants called when progress is made.
		/// </summary>
		private readonly CopyFileProgressCallback progressCallback;

		/// <summary>
		/// An object containing data supplied by the user.
		/// </summary>
		private readonly object userData;

		/// <summary>
		/// The DateTime when the file copy began.
		/// </summary>
		private DateTime timeStarted;

		/// <summary>
		/// Initializes a new instance of the CopyFileProgress class.
		/// </summary>
		/// <param name="sourceFilePath">The full path to the source file being copied.</param>
		/// <param name="destinationFilePath">The full path to the destination file being copied.</param>
		/// <param name="copyProgressCallback">A <see cref="CopyFileProgressCallback"/> method to call when progress is made.</param>
		/// <param name="userData">An object containing data to be used by the method.</param>
		internal CopyFileProgress(string sourceFilePath, string destinationFilePath, CopyFileProgressCallback copyProgressCallback, object userData)
		{
			// Save the input parameters (Validation should be done by CopyFileEx since this is an internal constructor)
			//
			this.SourceFilePath = sourceFilePath;
			this.DestinationFilePath = destinationFilePath;

			// It is OK for this to be null
			//
			this.progressCallback = copyProgressCallback;

			// Save the UserData object
			//
			this.userData = userData;
		}

		/// <summary>
		/// Gets the path to the destination file being copied.
		/// </summary>
		public string DestinationFilePath
		{
			get;
			private set;
		}

		/// <summary>
		/// Gets an estimated amount of time remaining until the copy operation will complete.
		/// </summary>
		public TimeSpan EstimatedTimeRemaining
		{
			get;
			private set;
		}

		/// <summary>
		/// Gets the percentage of the file that has been copied since the copy operation began.
		/// </summary>
		public decimal PercentComplete
		{
			get;
			private set;
		}

		/// <summary>
		/// Gets the path to the source file being copied.
		/// </summary>
		public string SourceFilePath
		{
			get;
			private set;
		}

		/// <summary>
		/// Gets the total size of the file, in bytes.
		/// </summary>
		public long TotalFileSize
		{
			get;
			private set;
		}

		/// <summary>
		/// Gets the total number of bytes transferred from the source file to the destination file since the copy operation began.
		/// </summary>
		public long TransferredBytes
		{
			get;
			private set;
		}

		/// <summary>
		/// A method to be passed as a callback to native methods that use the CopyProgressRoutine function.
		/// </summary>
		/// <param name="TotalFileSize">The total size of the file, in bytes.</param>
		/// <param name="TotalBytesTransferred">The total number of bytes transferred from the source file to the destination file since the copy operation began.</param>
		/// <param name="StreamSize">The total size of the current file stream, in bytes.</param>
		/// <param name="StreamBytesTransferred">The total number of bytes in the current stream that have been transferred from the source file to the destination file since the copy operation began.</param>
		/// <param name="dwStreamNumber">A handle to the current stream. The first time CopyProgressRoutine is called, the stream number is 1.</param>
		/// <param name="dwCallbackReason">The reason that CopyProgressRoutine was called.</param>
		/// <param name="hSourceFile">A handle to the source file.</param>
		/// <param name="hDestinationFile">A handle to the destination file.</param>
		/// <param name="lpData">Argument passed to CopyProgressRoutine by CopyFileEx, MoveFileTransacted, or MoveFileWithProgress.</param>
		/// <returns>The CopyProgressRoutine function should return one of the following values.
		/// CopyFileProgressAction.Cancel - Cancel the copy operation and delete the destination file.
		/// CopyFileProgressAction.Continue - Continue the copy operation.
		/// CopyFileProgressAction.Quiet - Continue the copy operation, but stop invoking CopyProgressRoutine to report progress.
		/// CopyFileProgressAction.Stop - Stop the copy operation. It can be restarted at a later time.</returns>
		internal CopyFileProgressAction CopyProgressNative(LARGE_INTEGER TotalFileSize, LARGE_INTEGER TotalBytesTransferred, LARGE_INTEGER StreamSize, LARGE_INTEGER StreamBytesTransferred, DWORD dwStreamNumber, CopyFileProgressCallbackReason dwCallbackReason, IntPtr hSourceFile, IntPtr hDestinationFile, IntPtr lpData)
		{
			// See if a user callback was specified
			//
			if (this.progressCallback == null)
			{
				// Tell CopyFileEx to stop updating progress
				//
				return CopyFileProgressAction.Quiet;
			}

			switch (dwCallbackReason)
			{
				case CopyFileProgressCallbackReason.StreamSwitch:  // Copy started
					{
						// Save the time the copy started
						//
						this.timeStarted = DateTime.Now;

						// Set total file size
						//
						this.TotalFileSize = (long)TotalFileSize;
					}
					break;

				case CopyFileProgressCallbackReason.ChunkFinished:  // A portion of the was copied
					{
						// Default the percent complete to 0
						//
						int percentComplete = 0;

						// See if the file has any content and that it isn't completely copied
						//
						if (TotalFileSize > 0 && TotalBytesTransferred < TotalFileSize)
						{
							// Calculate the percent complete rounded to the nearest tenth
							//
							percentComplete = (int)(TotalBytesTransferred / TotalFileSize);
						}

						// See if progress was made percent-wise
						//
						if (percentComplete != this.PercentComplete)
						{
							// Set transferred bytes
							//
							this.TransferredBytes = (long)TotalBytesTransferred;

							// Set percent complete
							//
							this.PercentComplete = percentComplete;

							// Calculate the estimated time remaining in seconds
							//
							this.EstimatedTimeRemaining = TimeSpan.FromSeconds((((DateTime.Now - this.timeStarted).TotalSeconds / (double)TotalBytesTransferred) * (TotalFileSize - TotalBytesTransferred)));

							// Execute the user's callback method and return their result
							//
							return this.progressCallback(this, this.userData);
						}
					}
					break;
			}
			return CopyFileProgressAction.Continue;
		}
	}
}